@@ -19,10 +19,12 @@ use Class::Accessor::Lite
         sql_builder
         sql_comment
         owner_pid
+        mode
+        fields_case
     )]
 ;
 
-our $VERSION = '0.14_04';
+our $VERSION = '0.14_05';
 
 sub load_plugin {
     my ($class, $pkg, $opt) = @_;
@@ -35,7 +37,7 @@ sub load_plugin {
           ( $opt->{alias} && $opt->{alias}->{$meth} )
           ? $opt->{alias}->{$meth}
           : $meth;
-        *{"${class}::${dest_meth}"} = *{"${pkg}::$meth"};
+        *{"${class}::${dest_meth}"} = $pkg->can($meth);
     }
 
     $pkg->init($pkg) if $pkg->can('init');
@@ -48,6 +50,8 @@ sub new {
     my $self = bless {
         schema_class => "$class\::Schema",
         owner_pid    => $$,
+        mode         => 'ping',
+        fields_case  => 'NAME_lc',
         %args,
     }, $class;
 
@@ -123,9 +127,7 @@ sub _on_connect_do {
 sub reconnect {
     my $self = shift;
 
-    if ($self->in_transaction_check) {
-        Carp::confess("Detected disconnected database during a transaction. Refusing to proceed");
-    }
+    $self->in_transaction_check;
 
     my $dbh = $self->{dbh};
 
@@ -146,6 +148,7 @@ sub reconnect {
 
         $self->owner_pid($$);
         $self->_on_connect_do;
+        $self->_prepare_from_dbh;
     }
 }
 
@@ -174,6 +177,9 @@ sub _prepare_from_dbh {
         $builder = Teng::QueryBuilder->new(driver => $self->{driver_name} );
         $self->sql_builder( $builder );
     }
+    $self->{dbh}->{FetchHashKeyName} = $self->{fields_case};
+
+    $self->{schema}->prepare_from_dbh($self->{dbh}) if $self->{schema};
 }
 
 sub _verify_pid {
@@ -183,7 +189,10 @@ sub _verify_pid {
         $self->reconnect;
     }
     elsif ( my $dbh = $self->{dbh} ) {
-        if ( !$dbh->FETCH('Active') || !$dbh->ping ) {
+        if ( !$dbh->FETCH('Active') ) {
+            $self->reconnect;
+        }
+        elsif ( $self->mode eq 'ping' && !$dbh->ping) {
             $self->reconnect;
         }
     }
@@ -196,35 +205,61 @@ sub dbh {
     $self->{dbh};
 }
 
+sub connected {
+    my $self = shift;
+    my $dbh = $self->{dbh};
+    return $self->owner_pid && $dbh->ping;
+}
+
 sub _execute {
     my ($self, $sql, $binds) = @_;
 
+    if ($ENV{TENG_SQL_COMMENT} || $self->sql_comment) {
+        my $i = 1; # optimize, as we would *NEVER* be called
+        while ( my (@caller) = caller($i++) ) {
+            next if ( $caller[0]->isa( __PACKAGE__ ) );
+            my $comment = "$caller[1] at line $caller[2]";
+            $comment =~ s/\*\// /g;
+            $sql = "/* $comment */\n$sql";
+            last;
+        }
+    }
+
     my $sth;
-    eval {
-        if ($ENV{TENG_SQL_COMMENT} || $self->sql_comment) {
-            my $i = 1; # optimize, as we would *NEVER* be called
-            while ( my (@caller) = caller($i++) ) {
-                next if ( $caller[0]->isa( __PACKAGE__ ) );
-                my $comment = "$caller[1] at line $caller[2]";
-                $comment =~ s/\*\// /g;
-                $sql = "/* $comment */\n$sql";
-                last;
+    eval { $sth = $self->__execute($sql, $binds) };
+
+    if ($@) {
+        if ( $self->mode eq 'fixup' ) {
+            if ( $self->connected ) {
+                $self->handle_error($sql, $binds, $@);
+            }
+            $self->reconnect;
+            eval { $sth = $self->__execute($sql, $binds) };
+            if ($@) {
+                $self->handle_error($sql, $binds, $@);
             }
         }
-        $sth = $self->dbh->prepare($sql);
-        my $i = 1;
-        for my $v ( @{ $binds || [] } ) {
-            $sth->bind_param( $i++, ref($v) ? @$v : $v );
+        else {
+            $self->handle_error($sql, $binds, $@);
         }
-        $sth->execute();
-    };
-    if ($@) {
-        $self->handle_error($sql, $binds, $@);
     }
 
     return $sth;
 }
 
+sub __execute {
+    my ($self, $sql, $binds) = @_;
+
+    my $sth = $self->dbh->prepare($sql);
+    my $i = 1;
+    for my $v ( @{ $binds || [] } ) {
+        $sth->bind_param( $i++, ref($v) ? @$v : $v );
+    }
+    $sth->execute();
+
+    return $sth;
+}
+
 sub _last_insert_id {
     my ($self, $table_name) = @_;
 
@@ -402,11 +437,42 @@ sub in_transaction_check {
 }
 
 sub txn_scope {
+    my $self = shift;
     my @caller = caller();
-    $_[0]->txn_manager->txn_scope(caller => \@caller);
+
+    my $scope;
+    if ( $self->mode eq 'fixup' ) {
+        eval { $scope = $self->txn_manager->txn_scope(caller => \@caller) };
+        if ( $@ ) {
+            if ( $self->connected ) {
+                die $@;
+            }
+            $self->reconnect;
+            $scope = $self->txn_manager->txn_scope(caller => \@caller);
+        }
+    }
+    else {
+        $scope = $self->txn_manager->txn_scope(caller => \@caller);
+    }
+    return $scope;
 }
 
-sub txn_begin    { $_[0]->txn_manager->txn_begin    }
+sub txn_begin {
+    my $self = shift;
+    if ( $self->mode eq 'fixup' ) {
+        eval { $self->txn_manager->txn_begin };
+        if ( $@ ) {
+            if ( $self->connected ) {
+                die $@;
+            }
+            $self->reconnect;
+            $self->txn_manager->txn_begin;
+        }
+    }
+    else {
+        $self->txn_manager->txn_begin;
+    }
+}
 sub txn_rollback { $_[0]->txn_manager->txn_rollback }
 sub txn_commit   { $_[0]->txn_manager->txn_commit   }
 sub txn_end      { $_[0]->txn_manager->txn_end      }
@@ -423,6 +489,15 @@ sub do {
     $ret;
 }
 
+sub _get_select_columns {
+    my ($self, $table, $opt) = @_;
+
+    return $opt->{'+columns'}
+        ? [@{$table->{escaped_columns}{$self->{driver_name}}}, @{$opt->{'+columns'}}]
+        : ($opt->{columns} || $table->{escaped_columns}{$self->{driver_name}})
+    ;
+}
+
 sub search {
     my ($self, $table_name, $where, $opt) = @_;
 
@@ -431,14 +506,9 @@ sub search {
         Carp::croak("No such table $table_name");
     }
 
-    my $columns = $opt->{'+columns'}
-        ? [@{$table->{columns}}, @{$opt->{'+columns'}}]
-        : ($opt->{columns} || $table->{columns})
-    ;
-
     my ($sql, @binds) = $self->{sql_builder}->select(
         $table_name,
-        $columns,
+        $self->_get_select_columns($table, $opt),
         $where,
         $opt
     );
@@ -474,19 +544,14 @@ sub single {
     my $table = $self->{schema}->get_table( $table_name );
     Carp::croak("No such table $table_name") unless $table;
 
-    my $columns = $opt->{'+columns'}
-        ? [@{$table->{columns}}, @{$opt->{'+columns'}}]
-        : ($opt->{columns} || $table->{columns})
-    ;
-
     my ($sql, @binds) = $self->{sql_builder}->select(
         $table_name,
-        $columns,
+        $self->_get_select_columns($table, $opt),
         $where,
         $opt
     );
     my $sth = $self->_execute($sql, \@binds);
-    my $row = $sth->fetchrow_hashref('NAME_lc');
+    my $row = $sth->fetchrow_hashref($self->{fields_case});
 
     return unless $row;
     return $row if $self->{suppress_row_objects};
@@ -512,6 +577,7 @@ sub search_by_sql {
         sth              => $sth,
         sql              => $sql,
         row_class        => $self->{schema}->get_row_class($table_name),
+        table            => $self->{schema}->get_table( $table_name ),
         table_name       => $table_name,
         suppress_object_creation => $self->{suppress_row_objects},
     );
@@ -687,6 +753,28 @@ You must pass C<connect_info> or C<dbh> to the constructor.
 
 Specifies the database handle to use. 
 
+=item * C<mode>
+
+=over
+
+=item * C<ping(default)>
+
+reconnect at dbh->ping fail each execute.
+
+=item * C<fixup>
+
+reconnect at fail execute.
+
+=item * C<no_ping>
+
+no auto reconnect.
+
+=item * C<fields_case>
+
+specific DBI.pm's FetchHashKeyName.
+
+=back
+
 =item * C<schema>
 
 Specifies the Teng::Schema instance to use.
